"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const luxon_1 = require("luxon");
const null_as_undefined_1 = require("null-as-undefined");
const ynab_1 = require("ynab");
const defaultOptions = {};
class YNAB {
    constructor(options) {
        const { accessToken } = { ...defaultOptions, ...options };
        this.api = new ynab_1.API(accessToken);
    }
    async listBudgets() {
        const response = await this.api.budgets.getBudgets();
        const { budgets } = response.data;
        return budgets.map(({ id }) => ({ id }));
    }
    async listAccountTransactions(budgetId, accountId, since) {
        const response = await this.api.transactions.getTransactionsByAccount(budgetId, accountId, since.toJSDate());
        const { transactions } = response.data;
        return transactions
            .filter(({ deleted }) => !deleted)
            .map(({ id, date, amount, payee_name, import_id }) => ({
            id,
            on: luxon_1.DateTime.fromISO(date)
                .toUTC(0, { keepLocalTime: true })
                .startOf("day"),
            payee: null_as_undefined_1.nullAsUndefined(payee_name),
            amount: amount / 10,
            importId: null_as_undefined_1.nullAsUndefined(import_id),
        }));
    }
    async selectNewAccountTransactions(budgetId, accountId, since, accountTransactions) {
        const budgetTransactions = await this.listAccountTransactions(budgetId, accountId, since);
        return accountTransactions.filter((accountTransaction) => {
            const accountTransactionImportId = this.makeImportId(accountTransaction);
            return !budgetTransactions.find(function matchExistingTransaction(budgetTransaction) {
                if (accountTransactionImportId === budgetTransaction.importId) {
                    return true;
                }
                const isSameDate = accountTransaction.on.equals(budgetTransaction.on);
                const isSamePayee = accountTransaction.payee.toLowerCase() ===
                    budgetTransaction.payee?.toLowerCase();
                const isSameAmount = accountTransaction.amount === budgetTransaction.amount;
                return isSameDate && isSamePayee && isSameAmount;
            });
        });
    }
    async createTransactions(budgetId, accountId, transactions) {
        const budgetTransactions = transactions.map((transaction) => {
            const amount = transaction.amount * 10;
            const date = transaction.on.toISO();
            return {
                account_id: accountId,
                date,
                amount,
                payee_name: transaction.payee,
                cleared: ynab_1.SaveTransaction.ClearedEnum.Cleared,
                import_id: this.makeImportId(transaction),
            };
        });
        const response = await this.api.transactions.createTransactions(budgetId, {
            transactions: budgetTransactions,
        });
        return response.data.transaction_ids.length;
    }
    async syncTransactions(budgetId, accountId, since, accountTransactions) {
        const newTransactions = await this.selectNewAccountTransactions(budgetId, accountId, since, accountTransactions);
        if (newTransactions.length === 0) {
            return 0;
        }
        return await this.createTransactions(budgetId, accountId, newTransactions);
    }
    makeImportId(transaction) {
        const amount = transaction.amount * 10;
        const date = transaction.on.toISO();
        return `BUDGET-SYNC:${amount}:${date.split("T")[0]}:${transaction.payee}`.substr(0, 34);
    }
}
exports.default = YNAB;
