"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.builder = void 0;
const open_1 = __importDefault(require("open"));
const polka_1 = __importDefault(require("polka"));
const monzo_1 = __importDefault(require("../../accounts/monzo"));
exports.builder = {
    "monzo-client-id": {
        string: true,
    },
    "monzo-client-secret": {
        string: true,
    },
    "monzo-refresh-token": {
        string: true,
    },
    quiet: {
        boolean: true,
        default: false,
        describe: "Suppress non-essential output",
    },
    refresh: {
        boolean: true,
        default: false,
        describe: "Refresh existing token instead of generating new one",
    },
};
const handler = async ({ monzoClientId, monzoClientSecret, monzoRefreshToken, quiet, refresh, }) => {
    if (!monzoClientId || !monzoClientSecret) {
        throw new Error("You must provide a client ID and secret to auth Monzo");
    }
    if (refresh) {
        if (!monzoRefreshToken) {
            throw new Error("You must provide a refresh token to refresh a Monzo access token");
        }
        await refreshAuth(monzoClientId, monzoClientSecret, monzoRefreshToken, quiet);
    }
    else {
        await auth(monzoClientId, monzoClientSecret, quiet);
    }
};
exports.handler = handler;
async function auth(clientId, clientSecret, quiet) {
    const port = 3333;
    const redirectUri = `http://localhost:${port}`;
    const { state, url: authUrl } = monzo_1.default.makeOAuthURL(clientId, redirectUri);
    quiet || console.log(`Opening your browser to: ${authUrl}`);
    await open_1.default(authUrl);
    const code = await new Promise((resolve, reject) => {
        const app = polka_1.default();
        app.get("/", (req, res) => {
            const isSuccess = req.query.state === state;
            if (isSuccess) {
                res.writeHead(200, { "Content-Type": "text/plain" });
                res.end("Success! Return to your terminal to continue...");
                quiet || console.log("Success!");
            }
            else {
                res.writeHead(401, { "Content-Type": "text/plain" });
                res.end("Invalid state!");
                quiet || console.log("Something went wrong.");
            }
            app.server.close(() => {
                isSuccess
                    ? resolve(req.query.code)
                    : reject(new Error("Unable to successfully auth with Monzo"));
            });
        });
        app.listen(port, (err) => {
            if (err) {
                throw err;
            }
            quiet ||
                console.log(`Waiting for response from Monzo on port ${port}...`);
        });
    });
    quiet || console.log("Requesting access token from Monzo...");
    const { accessToken, refreshToken } = await monzo_1.default.completeOAuth(clientId, clientSecret, redirectUri, code);
    quiet || console.log("Success!");
    console.log(`BUDGET_SYNC_MONZO_ACCESS_TOKEN=${accessToken}`);
    console.log(`BUDGET_SYNC_MONZO_REFRESH_TOKEN=${refreshToken}`);
    quiet || console.log("Now approve access in your Monzo app!");
}
async function refreshAuth(clientId, clientSecret, oldRefreshToken, quiet) {
    quiet || console.log("Refreshing access token with Monzo...");
    const { accessToken, refreshToken } = await monzo_1.default.refreshOAuth(clientId, clientSecret, oldRefreshToken);
    quiet || console.log("Success!");
    console.log(`BUDGET_SYNC_MONZO_ACCESS_TOKEN=${accessToken}`);
    console.log(`BUDGET_SYNC_MONZO_REFRESH_TOKEN=${refreshToken}`);
    quiet || console.log("Done!");
}
