"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.builder = exports.describe = exports.aliases = exports.command = void 0;
const luxon_1 = require("luxon");
const monzo_1 = __importDefault(require("../accounts/monzo"));
const ynab_1 = __importDefault(require("../budgets/ynab"));
exports.command = "run";
exports.aliases = ["start", "sync"];
exports.describe = "Synchronize configured accounts to budgets";
const builder = (args) => args
    .options({
    "days-to-sync": {
        describe: "Number of days in past to synchronize",
        default: 14,
        number: true,
    },
    "monzo-access-token": {
        string: true,
    },
    quiet: {
        boolean: true,
        default: false,
        describe: "Suppress non-essential output",
    },
    "ynab-access-token": {
        string: true,
    },
    "ynab-account-map": {
        describe: 'Stringified JSON specifying how YNAB budget accounts map to real accounts:\n\n{ "<budget_account_id>": "<account_provider>:<account_id>", ... }\n\n<budget_account_id> = ID of an account\'s internal representation in your budgeting service\n\n<account_provider> = name of module to use to communicate with your account (eg "monzo" for Monzo)\n\n<account_id> = account provider\'s ID for your account in their API',
        string: true,
    },
    "ynab-budget-id": {
        string: true,
    },
})
    .group(["days-to-sync", "help", "quiet"], "General options:")
    .group(["monzo-access-token"], "Monzo options:")
    .group(["ynab-access-token", "ynab-account-map", "ynab-budget-id"], "YNAB options:");
exports.builder = builder;
const handler = async ({ daysToSync, monzoAccessToken, quiet, ynabAccessToken, ynabBudgetId, ynabAccountMap, }) => {
    const since = luxon_1.DateTime.utc().startOf("day").minus({ days: daysToSync });
    const accountProviders = {};
    if (monzoAccessToken) {
        quiet || console.log("Initializing monzo provider...");
        accountProviders.monzo = new monzo_1.default({ accessToken: monzoAccessToken });
    }
    if (Object.keys(accountProviders).length === 0) {
        throw new Error("No account providers specified");
    }
    if (ynabAccessToken && ynabBudgetId && ynabAccountMap) {
        syncYnab(ynabAccessToken, ynabBudgetId, ynabAccountMap, accountProviders, since);
    }
    quiet || console.log("Done!");
};
exports.handler = handler;
async function syncYnab(accessToken, budgetId, rawAccountMap, accountProviders, since, quiet) {
    quiet || console.log("Initializing ynab provider...");
    const ynab = new ynab_1.default({ accessToken });
    quiet || console.log("Syncing ynab budget following the ynab account map...");
    const accountMap = JSON.parse(rawAccountMap);
    for (const [budgetAccountId, realAccountId] of Object.entries(accountMap)) {
        const [accountProviderName, accountId] = realAccountId.split(":");
        const accountProvider = accountProviders[accountProviderName];
        quiet || console.log(`Fetching ${accountProviderName} transactions...`);
        const accountTransactions = await accountProvider.listTransactions(accountId, since);
        quiet || console.log("Syncing transactions with ynab...");
        const count = await ynab.syncTransactions(budgetId, budgetAccountId, since, accountTransactions);
        quiet || console.log(`Synced ${count} transactions with ynab.`);
    }
}
